<?xml version="1.0"?>
<VOTABLE version="1.1" xsi:schemaLocation="http://www.ivoa.net/xml/VOTable/v1.1" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <INFO name="QUERY_STATUS" value="OK"/>
  <RESOURCE type="results">
    <TABLE utype="photdm:PhotometryFilter.transmissionCurve.spectrum">
    <PARAM name="FilterProfileService" value="ivo://svo/fps" ucd="meta.ref.ivorn" utype="PhotometryFilter.fpsIdentifier" datatype="char" arraysize="*"/>
    <PARAM name="filterID" value="Generic/Bessell.V" ucd="meta.id" utype="photdm:PhotometryFilter.identifier" datatype="char" arraysize="*"/>
    <PARAM name="WavelengthUnit" value="Angstrom" ucd="meta.unit" utype="PhotometryFilter.SpectralAxis.unit" datatype="char" arraysize="*"/>
    <PARAM name="WavelengthUCD" value="em.wl" ucd="meta.ucd" utype="PhotometryFilter.SpectralAxis.UCD" datatype="char" arraysize="*"/>
    <PARAM name="Description" value="Bessell B generic filter" ucd="meta.note" utype="photdm:PhotometryFilter.description" datatype="char" arraysize="*"/>
    <PARAM name="PhotSystem" value="Bessell" utype="photdm:PhotometricSystem.description" datatype="char" arraysize="*">
       <DESCRIPTION>Photometric system</DESCRIPTION>
    </PARAM>
    <PARAM name="DetectorType" value="0" utype="photdm:PhotometricSystem.detectorType" datatype="char" arraysize="*">
       <DESCRIPTION>Detector type. 0:Energy counter, 1:Photon counter.</DESCRIPTION>
    </PARAM>
    <PARAM name="Band" value="V" utype="photdm:PhotometryFilter.bandName" datatype="char" arraysize="*"/>
    <PARAM name="Description" value="Bessell B generic filter" ucd="meta.note" utype="photdm:PhotometryFilter.description" datatype="char" arraysize="*"/>
    <PARAM name="components" value="Filter" datatype="char" arraysize="*">
       <DESCRIPTION>Transmission components</DESCRIPTION>
    </PARAM>
    <PARAM name="WavelengthRef" value="5477.710624673" unit="Angstrom" ucd="em.wl" datatype="float" >
       <DESCRIPTION>Reference wavelength. Defined as the same than the pivot wavelength.</DESCRIPTION>
    </PARAM>
    <PARAM name="WavelengthMean" value="5512.1173634" unit="Angstrom" ucd="em.wl" utype="photdm:PhotometryFilter.SpectralAxis.Coverage.Location.Value" datatype="float" >
       <DESCRIPTION>Mean wavelength. Defined as integ[x*filter(x) dx]/integ[filter(x) dx]</DESCRIPTION>
    </PARAM>
    <PARAM name="WavelengthEff" value="5445.78567536" unit="Angstrom" ucd="em.wl.effective" datatype="float" >
       <DESCRIPTION>Effective wavelength. Defined as integ[x*filter(x)*vega(x) dx]/integ[filter(x)*vega(x) dx]</DESCRIPTION>
    </PARAM>
    <PARAM name="WavelengthMin" value="4733.33333333" unit="Angstrom" ucd="em.wl;stat.min" utype="photdm:PhotometryFilter.SpectralAxis.Coverage.Bounds.Start" datatype="float" >
       <DESCRIPTION>Minimum filter wavelength. Defined as the first lambda value with a transmission at least 1% of maximum transmission</DESCRIPTION>
    </PARAM>
    <PARAM name="WavelengthMax" value="6875" unit="Angstrom" ucd="em.wl;stat.max" utype="photdm:PhotometryFilter.SpectralAxis.Coverage.Bounds.Stop" datatype="float" >
       <DESCRIPTION>Maximum filter wavelength. Defined as the last lambda value with a transmission at least 1% of maximum transmission</DESCRIPTION>
    </PARAM>
    <PARAM name="WidthEff" value="893.08062963" unit="Angstrom" ucd="instr.bandwidth" utype="photdm:PhotometryFilter.SpectralAxis.Coverage.Bounds.Extent" datatype="float" >
       <DESCRIPTION>Effective width. Defined as integ[x*filter(x) dx].\nEquivalent to the horizontal size of a rectangle with height equal to maximum transmission and with the same area that the one covered by the filter transmission curve.</DESCRIPTION>
    </PARAM>
    <PARAM name="WavelengthCen" value="5439.26510196" unit="Angstrom" ucd="em.wl" datatype="float" >
       <DESCRIPTION>Central wavelength. Defined as the central wavelength between the two points defining FWMH</DESCRIPTION>
    </PARAM>
    <PARAM name="WavelengthPivot" value="5477.710624673" unit="Angstrom" ucd="em.wl" datatype="float" >
       <DESCRIPTION>Peak wavelength. Defined as sqrt{integ[x*filter(x) dx]/integ[filter(x) dx/x]}</DESCRIPTION>
    </PARAM>
    <PARAM name="WavelengthPeak" value="5300" unit="Angstrom" ucd="em.wl" datatype="float" >
       <DESCRIPTION>Peak wavelength. Defined as the lambda value with larger transmission</DESCRIPTION>
    </PARAM>
    <PARAM name="WavelengthPhot" value="5467.44659017" unit="Angstrom" ucd="em.wl" datatype="float" >
       <DESCRIPTION>Photon distribution based effective wavelength. Defined as integ[x^2*filter(x)*vega(x) dx]/integ[x*filter(x)*vega(x) dx]</DESCRIPTION>
    </PARAM>
    <PARAM name="FWHM" value="852.443247403" unit="Angstrom" ucd="instr.bandwidth" datatype="float" >
       <DESCRIPTION>Full width at half maximum. Defined as the difference between the two wavelengths for which filter transmission is half maximum</DESCRIPTION>
    </PARAM>
    <PARAM name="Fsun" value="184.17354946428" unit="erg/cm2/s/A" ucd="phot.flux.density" datatype="float" >
       <DESCRIPTION>Sun flux</DESCRIPTION>
    </PARAM>
    <PARAM name="PhotCalID" value="Generic/Bessell.V/Vega" ucd="meta.id" utype="photdm:PhotCal.identifier" datatype="char" arraysize="*"/>
    <PARAM name="MagSys" value="Vega" ucd="meta.code" utype="photdm:PhotCal.MagnitudeSystem.type" datatype="char" arraysize="*"/>
    <PARAM name="ZeroPoint" value="3579.76114152" unit="Jy" ucd="phot.flux.density" utype="photdm:PhotCal.ZeroPoint.Flux.value" datatype="float" />
    <PARAM name="ZeroPointUnit" value="Jy" ucd="meta.unit" utype="photdm:PhotCal.ZeroPoint.Flux.unit" datatype="char" arraysize="*"/>
    <PARAM name="ZeroPointType" value="Pogson" ucd="meta.code" utype="photdm:PhotCal.ZeroPoint.type" datatype="char" arraysize="*"/>
      <FIELD name="Wavelength" utype="spec:Data.SpectralAxis.Value" ucd="em.wl" unit="Angstrom" datatype="float"/>
      <FIELD name="Transmission" utype="spec:Data.FluxAxis.Value" ucd="phys.transmission" unit="" datatype="float"/>
      <DATA>
        <TABLEDATA>
          <TR>
            <TD>4700.0</TD>
            <TD>0.0000000000</TD>
          </TR>
          <TR>
            <TD>4800.0</TD>
            <TD>0.0300000000</TD>
          </TR>
          <TR>
            <TD>4900.0</TD>
            <TD>0.1630000000</TD>
          </TR>
          <TR>
            <TD>5000.0</TD>
            <TD>0.4580000000</TD>
          </TR>
          <TR>
            <TD>5100.0</TD>
            <TD>0.7800000000</TD>
          </TR>
          <TR>
            <TD>5200.0</TD>
            <TD>0.9670000000</TD>
          </TR>
          <TR>
            <TD>5300.0</TD>
            <TD>1.0000000000</TD>
          </TR>
          <TR>
            <TD>5400.0</TD>
            <TD>0.9730000000</TD>
          </TR>
          <TR>
            <TD>5500.0</TD>
            <TD>0.8980000000</TD>
          </TR>
          <TR>
            <TD>5600.0</TD>
            <TD>0.7920000000</TD>
          </TR>
          <TR>
            <TD>5700.0</TD>
            <TD>0.6840000000</TD>
          </TR>
          <TR>
            <TD>5800.0</TD>
            <TD>0.5740000000</TD>
          </TR>
          <TR>
            <TD>5900.0</TD>
            <TD>0.4610000000</TD>
          </TR>
          <TR>
            <TD>6000.0</TD>
            <TD>0.3590000000</TD>
          </TR>
          <TR>
            <TD>6100.0</TD>
            <TD>0.2700000000</TD>
          </TR>
          <TR>
            <TD>6200.0</TD>
            <TD>0.1970000000</TD>
          </TR>
          <TR>
            <TD>6300.0</TD>
            <TD>0.1350000000</TD>
          </TR>
          <TR>
            <TD>6400.0</TD>
            <TD>0.0810000000</TD>
          </TR>
          <TR>
            <TD>6500.0</TD>
            <TD>0.0450000000</TD>
          </TR>
          <TR>
            <TD>6600.0</TD>
            <TD>0.0250000000</TD>
          </TR>
          <TR>
            <TD>6700.0</TD>
            <TD>0.0170000000</TD>
          </TR>
          <TR>
            <TD>6800.0</TD>
            <TD>0.0130000000</TD>
          </TR>
          <TR>
            <TD>6900.0</TD>
            <TD>0.0090000000</TD>
          </TR>
          <TR>
            <TD>7000.0</TD>
            <TD>0.0000000000</TD>
          </TR>
        </TABLEDATA>
      </DATA>
    </TABLE>
  </RESOURCE>
</VOTABLE>
